=== modified file 'NEWS'
--- NEWS	2005-12-01 05:09:27 +0000
+++ NEWS	2005-12-02 04:44:19 +0000
@@ -147,6 +147,9 @@
       If adding a new test script please add that to
       bzrlib.tests.blackbox.__init__. (Robert Collins)
 
+    * Much better error message if one of the test suites can't be 
+      imported.  (Martin Pool)
+
   INTERNALS:
 
     * WorkingTree.pull has been split across Branch and WorkingTree,

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-12-02 05:25:53 +0000
+++ bzrlib/tests/__init__.py	2005-12-02 05:42:35 +0000
@@ -557,14 +557,6 @@
         self.assertEqualDiff(content, open(path, 'r').read())
         
 
-class MetaTestLog(TestCase):
-    def test_logging(self):
-        """Test logs are captured when a test fails."""
-        self.log('a test message')
-        self._log_file.flush()
-        self.assertContainsRe(self._get_log(), 'a test message\n')
-
-
 def filter_suite_by_re(suite, pattern):
     result = TestUtil.TestSuite()
     filter_re = re.compile(pattern)
@@ -612,11 +604,7 @@
 
     global MODULES_TO_DOCTEST
 
-    # FIXME: If these fail to load, e.g. because of a syntax error, the
-    # exception is hidden by unittest.  Sucks.  Should either fix that or
-    # perhaps import them and pass them to unittest as modules.
-    testmod_names = \
-                  ['bzrlib.tests.MetaTestLog',
+    testmod_names = [ \
                    'bzrlib.tests.test_api',
                    'bzrlib.tests.test_gpg',
                    'bzrlib.tests.test_identitymap',
@@ -666,17 +654,25 @@
                    'bzrlib.tests.test_trace',
                    'bzrlib.tests.test_rio',
                    'bzrlib.tests.test_msgeditor',
+                   'bzrlib.tests.test_selftest',
                    ]
 
-    TestCase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%-30s %s' % ('bzr binary', TestCase.BZRPATH)
+    print '%10s: %s' % ('bzr', os.path.realpath(sys.argv[0]))
+    print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
     print
     suite = TestSuite()
-    suite.addTest(TestLoader().loadTestsFromNames(testmod_names))
+    # python2.4's TestLoader.loadTestsFromNames gives very poor 
+    # errors if it fails to load a named module - no indication of what's
+    # actually wrong, just "no such module".  We should probably override that
+    # class, but for the moment just load them ourselves. (mbp 20051202)
+    loader = TestLoader()
+    for mod_name in testmod_names:
+        mod = _load_module_by_name(mod_name)
+        suite.addTest(loader.loadTestsFromModule(mod))
     for package in packages_to_test():
         suite.addTest(package.test_suite())
     for m in MODULES_TO_TEST:
-        suite.addTest(TestLoader().loadTestsFromModule(m))
+        suite.addTest(loader.loadTestsFromModule(m))
     for m in (MODULES_TO_DOCTEST):
         suite.addTest(DocTestSuite(m))
     for p in bzrlib.plugin.all_plugins:
@@ -684,3 +680,13 @@
             suite.addTest(p.test_suite())
     return suite
 
+
+def _load_module_by_name(mod_name):
+    parts = mod_name.split('.')
+    module = __import__(mod_name)
+    del parts[0]
+    # for historical reasons python returns the top-level module even though
+    # it loads the submodule; we need to walk down to get the one we want.
+    while parts:
+        module = getattr(module, parts.pop(0))
+    return module

=== added file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_selftest.py	2005-12-02 04:44:19 +0000
@@ -0,0 +1,42 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the test framework
+"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCase, _load_module_by_name
+
+
+class SelftestTests(TestCase):
+
+    def test_import_tests(self):
+        mod = _load_module_by_name('bzrlib.tests.test_selftest')
+        self.assertEqual(mod.SelftestTests, SelftestTests)
+
+    def test_import_test_failure(self):
+        self.assertRaises(ImportError,
+                          _load_module_by_name,
+                          'bzrlib.no-name-yet')
+
+
+class MetaTestLog(TestCase):
+    def test_logging(self):
+        """Test logs are captured when a test fails."""
+        self.log('a test message')
+        self._log_file.flush()
+        self.assertContainsRe(self._get_log(), 'a test message\n')

